/*
 * linux/drivers/pcmcia/pxa/korebot.c
 *
 * Author:	Pierre Bureau & Stephane Magnenat
 * Created:	May 13, 2002
 * Copyright:	KTeam & LSA 2 EPFL
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Originally based upon linux/drivers/pcmcia/sa1100_neponset.c
 *
 * Korebot PCMCIA specific routines.
 *
 */

#include <linux/kernel.h>
#include <linux/sched.h>

#include <pcmcia/ss.h>

#include <asm/delay.h>
#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/arch/pcmcia.h>
#include <asm/arch/irqs.h>

#if defined(CONFIG_ARCH_KOREBOT_KTEAM)
/* GPIO Settings for K-TEAM Korebot Board Rev C3 */
#define KOREBOT_CD0		5
#define KOREBOT_CD1		4
#define KOREBOT_RDY_BUSY0	13
#define KOREBOT_RDY_BUSY1	14
#define KOREBOT_RST0 		19
#define KOREBOT_RST1 		21
#endif /* defined(CONFIG_ARCH_KOREBOT_KTEAM) */

#if defined(CONFIG_ARCH_KOREBOT_LE)
/* GPIO Settings for K-TEAM Korebot Board Rev C3 */
#define KOREBOT_CD0		5
#define KOREBOT_CD1		4
#define KOREBOT_RDY_BUSY0	13
#define KOREBOT_RDY_BUSY1	14
#define KOREBOT_RST0 		19
#define KOREBOT_RST1 		21
#endif /* defined(CONFIG_ARCH_KOREBOT_LE) */

#if defined(CONFIG_ARCH_KOREBOT_SBOT)
/* GPIO Settings for LSA S-Bot Rev ? (15-oct-2003) */
#define KOREBOT_CD0		5
#define KOREBOT_CD1		4
#define KOREBOT_RDY_BUSY0	6
#define KOREBOT_RDY_BUSY1	13
#define KOREBOT_RST0		19
#define KOREBOT_RST1		21
#endif /* defined(CONFIG_ARCH_KOREBOT_SBOT) */

#define GPIO_READ(gpio) (( (gpio) < 32 ? GPLR0 >> (gpio) : \
		((gpio) < 64 ? GPLR1 >> ((gpio)-32) : (GPLR2 >> ((gpio)-64)))) & 1)


static int korebot_pcmcia_init(struct pcmcia_init *init)
{
  int return_val=0;

  printk( "Korebot CF Init\n");

  /* Output, Enable Reset */
  GPDR(KOREBOT_RST0) |= GPIO_bit(KOREBOT_RST0);
  GPDR(KOREBOT_RST1) |= GPIO_bit(KOREBOT_RST1);
  /* Clear Reset (Reset Low => not active) */
  GPCR(KOREBOT_RST0) = GPIO_bit(KOREBOT_RST0);
  GPCR(KOREBOT_RST1) = GPIO_bit(KOREBOT_RST1);

  if (init->handler)
  {
	/* Card detect */
	/* set IO to input and interrupt to falling edge */
	GPDR(KOREBOT_CD0) &= ~GPIO_bit(KOREBOT_CD0);
	set_GPIO_IRQ_edge(KOREBOT_CD0, GPIO_BOTH_EDGES);
	return_val+=request_irq(IRQ_GPIO(KOREBOT_CD0), init->handler, SA_INTERRUPT,
				"Korebot CompactFlash (0) CD", NULL);
#ifndef CONFIG_ARCH_KOREBOT_LE
	GPDR(KOREBOT_CD1) &= ~GPIO_bit(KOREBOT_CD1);
	set_GPIO_IRQ_edge(KOREBOT_CD1, GPIO_BOTH_EDGES);
	return_val+=request_irq(IRQ_GPIO(KOREBOT_CD1), init->handler, SA_INTERRUPT,
                                "Korebot CompactFlash (1) CD", NULL);
#endif

          /* BVD */
        /* BVD pins are not connected on korebot */
        //GPDR(KOREBOT_BVD1) &= ~GPIO_bit(KOREBOT_BVD1);
	//GPDR(KOREBOT_BVD2) &= ~GPIO_bit(KOREBOT_BVD2);
  }
  else
    printk("Korebot CF : Null handler passed\n");

  /* Card ready */
  /* set IO to input and interrupt to falling edge */
  GPDR(KOREBOT_RDY_BUSY0) &= ~GPIO_bit(KOREBOT_RDY_BUSY0);
  set_GPIO_IRQ_edge(KOREBOT_RDY_BUSY0, GPIO_FALLING_EDGE);
#ifndef CONFIG_ARCH_KOREBOT_LE
  GPDR(KOREBOT_RDY_BUSY1) &= ~GPIO_bit(KOREBOT_RDY_BUSY1);
  set_GPIO_IRQ_edge(KOREBOT_RDY_BUSY1, GPIO_FALLING_EDGE);
#endif

#ifdef CONFIG_ARCH_KOREBOT_LE
  return (return_val<0) ? -1 : 1;
#else
  return (return_val<0) ? -1 : 2;
#endif
}

static int korebot_pcmcia_shutdown(void)
{
  /* Free ressources */
  free_irq(IRQ_GPIO(KOREBOT_CD0), NULL);
  free_irq(IRQ_GPIO(KOREBOT_CD1), NULL);

  return 0;
}

static int korebot_pcmcia_socket_state(struct pcmcia_state_array
		*state_array)
{
	int return_val=1;	/* 1 for IRQ pending */

#ifdef CONFIG_ARCH_KOREBOT_LE
	if(state_array->size<1) return -1;
#else
	if(state_array->size<2) return -1;
#endif

	memset(state_array->state, 0,
			(state_array->size)*sizeof(struct pcmcia_state));

	state_array->state[0].detect=!GPIO_READ(KOREBOT_CD0);
	state_array->state[0].ready=GPIO_READ(KOREBOT_RDY_BUSY0);
	/* we don't have bvd, we are never write-protected, we are always at 3V */
	state_array->state[0].bvd1=1;//GPIO_READ(KOREBOT_BVD1);
	state_array->state[0].bvd2=1;//GPIO_READ(KOREBOT_BVD2);
	state_array->state[0].wrprot=0;
	state_array->state[0].vs_3v=1;
	state_array->state[0].vs_Xv=0;

#ifndef CONFIG_ARCH_KOREBOT_LE
	state_array->state[1].detect=!GPIO_READ(KOREBOT_CD1);
	state_array->state[1].ready=GPIO_READ(KOREBOT_RDY_BUSY1);
	/* we don't have bvd, we are never write-protected, we are always at 3V */
	state_array->state[1].bvd1=1;
	state_array->state[1].bvd2=1;
	state_array->state[1].wrprot=0;
	state_array->state[1].vs_3v=1;
	state_array->state[1].vs_Xv=0;
#endif

	return return_val;
}

static int korebot_pcmcia_get_irq_info(struct pcmcia_irq_info *info)
{
	printk("%s called\n", __FUNCTION__);
	switch (info->sock)
	{
	case 0 :
		info->irq=IRQ_GPIO(KOREBOT_RDY_BUSY0);
		printk("%s(): irq for socket 0 is %u\n",
				__FUNCTION__, info->irq);
		break;

	case 1:
		info->irq=IRQ_GPIO(KOREBOT_RDY_BUSY1);
		printk("%s(): irq for socket 1 is %u\n",
				__FUNCTION__, info->irq);
		break;

	default:
		return -1;
	}

	return 0;
}

static int
korebot_pcmcia_configure_socket(unsigned int sock, socket_state_t *state)
{
	unsigned long flags;
	int ret=1;

	local_irq_save(flags);

	printk(KERN_NOTICE "%s(): Requesting Vcc %u and Vpp %u for socket %u.\n",
							__FUNCTION__, state->Vcc, state->Vpp, sock);

	switch (sock)
	{
		case 0 :
			/*
			switch (state->Vcc)
			{
				case 0:
				//GPSR(KOREBOT_RST0) = GPIO_bit(KOREBOT_RST0);
				break;

				case 33:
				//GPCR(KOREBOT_RST0) = GPIO_bit(KOREBOT_RST0);
				break;

				default:
				break;
			}
			*/
			if (state->flags & SS_RESET)
			{
				printk(KERN_NOTICE "%s(): Reset on sock 0\n", __FUNCTION__);
				GPSR(KOREBOT_RST0) = GPIO_bit(KOREBOT_RST0);
			}
			else
			{
				printk(KERN_NOTICE "%s(): Reset off sock 0\n", __FUNCTION__);
				GPCR(KOREBOT_RST0) = GPIO_bit(KOREBOT_RST0);
			}
		break;

		case 1:
		/*
			switch (state->Vcc)
			{
				case 0:
				//GPSR(KOREBOT_RST1) = GPIO_bit(KOREBOT_RST1);
				break;

				case 33:
				//GPCR(KOREBOT_RST1) = GPIO_bit(KOREBOT_RST1);
				break;

				default:
				break;
			}
			*/
			if (state->flags & SS_RESET)
			{
				printk(KERN_NOTICE "%s(): Reset on sock 1\n", __FUNCTION__);
				GPSR(KOREBOT_RST1) = GPIO_bit(KOREBOT_RST1);
			}
			else
			{
				printk(KERN_NOTICE "%s(): Reset off sock 1\n", __FUNCTION__);
				GPCR(KOREBOT_RST1) = GPIO_bit(KOREBOT_RST1);
			}
		break;

		default:

		break;
	}
	udelay(100);
	local_irq_restore(flags);
	return ret;
}

struct pcmcia_low_level korebot_pcmcia_ops = {
  korebot_pcmcia_init,
  korebot_pcmcia_shutdown,
  korebot_pcmcia_socket_state,
  korebot_pcmcia_get_irq_info,
  korebot_pcmcia_configure_socket
};
