/*--------------------------------------------------------------------
 * pxa_ssi.c - SSI bus driver for PXA 25x 
 *--------------------------------------------------------------------
 * $Id: pxa_ssi.c,v 1.1.1.1 2006/10/31 17:13:43 pbureau Exp $
 *--------------------------------------------------------------------
 * $Author: pbureau $
 * $Revision: 1.1.1.1 $
 * $Date: 2006/10/31 17:13:43 $
 *--------------------------------------------------------------------
 * Contact: cgaudin@k-team.com
 *          http://www.k-team.com
 *--------------------------------------------------------------------*/

#include <linux/types.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/sched.h>
#include <linux/delay.h>
#ifdef CONFIG_DEVFS_FS 
#include <linux/devfs_fs_kernel.h>
#endif /* CONFIG_DEVFS_FS */

#include <asm/mach-types.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/arch/pxa-regs.h>


#include "ssi_bus.h"
#include "ssi_dev.h"

/* Interrupt Handler
 */
static void pxa_ssi_intr( int irq , void * dev_id , struct pt_regs * regs )
{
  struct ssi_bus *bus = (struct ssi_bus *)dev_id;
  u_int data[16];
  u_int val;
  u_int level;
  int i;

  //printk("pxa_ssi_intr\n");

  val = SSSR;
  //printk("pxa_ssi: SSSR=%X\n" , val);

  if ( val & SSSR_RFS ) {
   
    if ( val & SSSR_RNE ) {
      level = (val << 12) & 15;
      for (i=0; i<level; i++) 
	data[i] = (SSDR & 0xffff);
      
      ssi_core_rcv( bus , data , level );
    }
  }
}

/* Select a device
 */
static int pxa_ssi_select_id( int id )
{
  return 0;
}

/* 
 * Select the specified device. 
 *
 * The upper layers will have already checked that the bus transmit 
 * queue is idle. 
 * We need to make sure that the interface itself is idle
 */
static int pxa_ssi_select( struct ssi_bus * bus , struct ssi_dev * dev )
{
  u_int val;

  printk("pxa_ssi_select\n");

  do {
    val = SSSR;
  }
  while (val & SSSR_BSY);

  /* 
   * Select a Device (only a single device can be present) 
   */ 
  if ( dev ) {
    /* selected device parameters */
    
    pxa_ssi_select_id( dev->id );

    bus->cfglen   = dev->cfglen;
    bus->framelen = dev->framelen;
    bus->clkpol   = dev->clkpol;
    bus->clkpha   = dev->clkpha;
    bus->proto    = dev->proto;
    bus->clkfreq  = dev->clkfreq;
  }
  else {
    /* default parameters */
    
    bus->cfglen   = 8;
    bus->framelen = 8;
    bus->clkpol   = 0;
    bus->clkpha   = 0;
    bus->proto    = SSI_SPI;
    bus->clkfreq  = (3686400/2); /* max freq */
  }
  
  val = SSCR_SSE;
  val |= SSCR_DSS( bus->framelen );
  
  switch( bus->proto ) {
  case SSI_SPI:       val |= SSCR_SPI; break;
  case SSI_MICROWIRE: val |= SSCR_MW;  break;
  case SSI_TISSF:     val |= SSCR_SSP; break;
  default:  
    printk("pxa_ssi: unknown protocol ! (bus->proto=%u) \n" , bus->proto);
    return -1; /* bad protocol */
  }
  
  /* We use the internal 3.6864 MHz clock */
  val |= SSCR_SCR(((3686400/ (2*(bus->clkfreq)))-1));
  
  SSCR0 = val;
  
  printk("pxa_ssi: SSCR0=%X\n", SSCR0);

  val = (SSCR_RIE|SSCR_RFT(3));
  
  if ( bus->clkpol )
    val |= SSCR_SPO;
  
  if ( bus->clkpha )
    val |= SSCR_SPH;
  
  if ( bus->cfglen != 8 && bus->cfglen != 16 ) {
    printk("pxa_ssi: bad length ! (bus->cfglen=%u) \n" , bus->cfglen);
    return -1; /* bad format */
  }

  if ( bus->cfglen == 16 )
    val |= SSCR_MWDS;
  
  SSCR1 = val;

  printk("pxa_ssi: SSCR1=%X\n", SSCR1);

  return 0;
}

/* Enable transmission and/or of some bytes
 */
static int pxa_ssi_trans( struct ssi_bus * bus , 
			  const u_int *buf , size_t len )
{
  u_int val;
  u_int i;

  printk("pxa_ssi_trans\n");

#ifdef CONFIG_SSI_KNET
  /* missing code */
#else
  for (i=0; i<len; i++) {
    val = SSSR;
    printk("pxa_ssi: SSSR=0x%X \n", val);
    if ( val & SSSR_TNF ) {
      SSDR = buf[i];
    }
    else
      return -EFAULT; 
  }
#endif /* CONFIG_SSI_KNET */
  return 0;
}

/* Initialize the SSP bus 
 */
static int pxa_ssi_bus_init( struct ssi_bus * bus )
{
  int rc;
 
  printk("pxa_ssi: initialize PXA SSI\n");

  if ((rc=request_irq( IRQ_SSP , pxa_ssi_intr, 0 , "pxa_ssi" , bus))) {
    printk("pxa_ssi: failed to request interrupt !\n");
    return rc; 
  }

  pxa_ssi_select( bus , NULL );

  return 0;
}


/* Deinitialize the SSP bus 
 */
static void pxa_ssi_bus_exit( struct ssi_bus * bus )
{
  printk("pxa_ssi_bus_exit\n");

  pxa_ssi_select( bus , NULL );

  /* Disable SSP */
  SSCR1 = 0;
  SSCR0 &= ~SSCR_SSE;

  free_irq( IRQ_SSP , bus );
}

static struct ssi_bus pxa_ssi_bus = {
  name:   "PXA SSI" ,
  init:   pxa_ssi_bus_init ,
  exit:   pxa_ssi_bus_exit ,
  select: pxa_ssi_select ,
  trans:  pxa_ssi_trans ,
};

static int __init pxa_ssi_init(void)
{
  return ssi_register_bus(&pxa_ssi_bus);
}

static void __exit pxa_ssi_exit(void)
{
 
  ssi_unregister_bus(&pxa_ssi_bus);
}

module_init(pxa_ssi_init);
module_exit(pxa_ssi_exit);

MODULE_AUTHOR("Cedric Gaudin");
MODULE_DESCRIPTION("PXA 25x SSI Driver");
MODULE_LICENSE("GPL");

