/*
 *  linux/drivers/ssi/pxa_spi.c
 *
 * This file provides a common framework to allow multiple SSI devices
 * to work together on a single SSI bus.
 *
 * You can use this in two ways:
 *  1. select the device, queue up data, flush the data to the device,
 *     (optionally) purge the received data, deselect the device.
 *  2. select the device, queue up one data word, flush to the device
 *     read data word, queue up next data word, flush to the device...
 *     deselect the device.
 */
#include <linux/types.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/fs.h>

#ifdef CONFIG_DEVFS_FS
#include <linux/devfs_fs_kernel.h>
#endif /* CONFIG_DEVFS_FS */

#include <asm/uaccess.h>
#include <asm/errno.h>

#include "ssi_bus.h"
#include "ssi_dev.h"

static u_int            ssi_bus_count = 0;
static struct ssi_bus   *ssi_busses[SSI_MAX_BUSSES];
#ifdef CONFIG_DEVFS_FS
static devfs_handle_t   ssi_devfs_handle;
#endif /* CONFIG_DEVFS_FS */

#define SSI_TIMEOUT (HZ / 10)

#define DEBUG

/**
 *	ssi_core_rcv - pass received SSI data to the device
 *	@bus: the bus that the data was received from
 *	@data: the data word that was received
 *
 *	This function is intended to be called by SSI bus device
 *	drivers to pass received data to the device driver.
 */
int ssi_core_rcv(struct ssi_bus *bus, u_int *data , u_int length )
{
  struct ssi_dev *dev = bus->dev;
  unsigned long flags;
  u_int pos;

  printk("ssi_core_rcv\n");

  if (dev && dev->rcv) {
    for ( pos = 0; pos < length ; pos++ ) {
      dev->rcv(dev, data[pos]);
    }
  }
  else {
    spin_lock_irqsave(&bus->lock,flags);
    for ( pos=0; pos<length; pos++ ) {
      if ( bus->rx_pos < bus->rx_size ) {
	bus->rx_buf[bus->rx_pos] = data[pos];
	bus->rx_pos++;
      }
      else
	break;
    }
    spin_unlock_irqrestore(&bus->lock,flags);
  }
  return 0;
}

/**
 *	ssi_transmit_data - queue SSI data for later transmission
 *	@dev: device requesting data to be transmitted
 *	@data: data word to be transmitted.
 *
 *	Queue one data word of SSI data for later transmission.
 */
int ssi_transmit_data( struct ssi_dev *dev , u_int data )
{
  struct ssi_bus *bus = dev->bus;

  /*
   * Make sure that we currently own the bus
   */
  if (bus->dev != dev)
    BUG();
  
  bus->trans( bus , &data , 1 );

  return 0;
}

/**
 *	ssi_select_device - select a SSI device for later transactions
 *	@dev: device to be selected
 */
int ssi_select_device( struct ssi_bus *bus , struct ssi_dev *dev )
{
  int retval;

#ifdef DEBUG
  printk("SSI: selecting device %s on bus %s\n",
	 dev ? dev->name : "<none>", bus->name);
#endif

  /*
  * Select the device if it wasn't already selected.
   */
  retval = 0;
  if (bus->dev != dev) {
    retval = bus->select(bus, dev);
    bus->dev = dev;
  }
  
  return retval;
}

/**
 *	ssi_register_device - register a SSI device with a SSI bus
 *	@bus: bus
 *	@dev: SSI device
 */
int ssi_register_device(struct ssi_bus *bus, struct ssi_dev *dev)
{
  int retval;
  
  dev->bus = bus;
  bus->devices++;
  retval = dev->init(dev);
  if (retval != 0) {
    dev->bus = NULL;
    bus->devices--;
  } else {
#ifdef DEBUG
    printk("SSI: registered new device %s on bus %s\n", dev->name, bus->name);
#endif
  }
  return retval;
}

/**
 *	ssi_unregister_device - unregister a SSI device from a SSI bus
 *	@dev: SSI device
 */
int ssi_unregister_device(struct ssi_dev *dev)
{
	struct ssi_bus *bus = dev->bus;

	if (bus->dev == dev)
		bus->dev = NULL;

	dev->bus = NULL;
	bus->devices--;
#ifdef DEBUG
	printk("SSI: unregistered device %s on bus %s\n", dev->name, bus->name);
#endif
	return 0;
}

/* Open a bus device 
 */
static int ssi_bus_open( struct inode * inode , struct file * filep )
{  
  unsigned int minor = MINOR(inode->i_rdev);

  if ( minor >= SSI_MAX_BUSSES || !ssi_busses[minor] ) {
    printk("SSI: no bus ssi%d\n" , minor);
    return -ENODEV;
  }

  filep->private_data = (void *)ssi_busses[minor];

  MOD_INC_USE_COUNT;
  printk("SSI: bus opened ssi%d\n" , minor);

  return 0;
}

/*  Close a bus device
 */
static int ssi_bus_release( struct inode * inode , struct file * filep )
{
  unsigned int minor = MINOR(inode->i_rdev);

  filep->private_data = 0;
  printk("SSI: bus closed ssi%d\n" , minor);

  MOD_DEC_USE_COUNT;

  return 0;
}

/* Write data on a bus device
 */
static ssize_t ssi_bus_write( struct file * filep , 
			      const char * buf , size_t len , loff_t * ppos )
{
  struct ssi_bus * bus;
  u_int          * tmp;
  int              ret = -1;
  u_long           flags;

  printk("SSI: ssi_bus_write\n");

  bus = (struct ssi_bus *)filep->private_data;

  tmp = kmalloc( sizeof(u_int)*len , GFP_KERNEL );
  if (tmp == NULL)
    return -ENOMEM;

  copy_from_user(tmp,buf,len);
  
  interruptible_sleep_on_timeout(&bus->waitq , SSI_TIMEOUT );

ret = bus->trans( bus , tmp , len );
  kfree(tmp);

  return ret;
}
    
/* Read data from a bus device
 */
static ssize_t ssi_bus_read( struct file * filep ,
			     char * buf , size_t len , loff_t * ppos )
{
  struct ssi_bus * bus;
  size_t           cnt;
  int              ret;

  printk("SSI: ssi_bus_read\n");

  bus = (struct ssi_bus *)filep->private_data;

  /* check_hw_buffer */

  /* read RX buffer */
  spin_lock_irq(&bus->lock);
  cnt = len;
  if (len > bus->rx_pos )
      cnt = bus->rx_pos;

  ret = copy_to_user(buf,bus->rx_buf,cnt);
  bus->rx_pos = 0;
  spin_unlock_irq(&bus->lock);
  
  return ( ret > 0 ? -EFAULT : cnt );
}


#ifdef CONFIG_DEVFS_FS
static struct file_operations ssi_bus_fops = {
  owner:   THIS_MODULE ,
  read:    ssi_bus_read ,
  write:   ssi_bus_write ,
  open:    ssi_bus_open ,
  release: ssi_bus_release 
};
#endif /* CONFIG_DEVFS_FS */

/**
 *	ssi_register_bus - register a SSI bus driver
 *	@bus: bus
 */
int ssi_register_bus(struct ssi_bus *bus)
{
  int  ret;
  char name_buf[16];

  init_waitqueue_head( &bus->waitq );
 
  bus->rx_size = SSI_BUFSIZE;
  bus->rx_pos  = 0;
  
  bus->tx_size = SSI_BUFSIZE;
  bus->tx_pos  = 0;

  ret = bus->init(bus);
  if (!ret) {
    bus->devices = 0;
#ifdef CONFIG_DEVFS_FS
    sprintf(name_buf, "bus%u", ssi_bus_count );
    bus->handle = devfs_register( ssi_devfs_handle , name_buf ,
				  DEVFS_FL_NONE , SSI_MAJOR , ssi_bus_count ,
				  S_IFCHR , &ssi_bus_fops , NULL );
#endif /* CONFIG_DEVFS_FS */
#ifdef DEBUG
    printk("SSI: registered new bus %s\n", bus->name);
#endif
    MOD_INC_USE_COUNT;
    bus->minor = ssi_bus_count;
    ssi_busses[ssi_bus_count]=bus;
    ssi_bus_count++;
  }
  
  return ret;
}

/**
 *	ssi_unregister_bus - unregister a SSI bus driver
 *	@bus: bus
 */
int ssi_unregister_bus(struct ssi_bus *bus)
{
  int ret = -EBUSY;
  
  wake_up_interruptible(&bus->waitq);

  if (bus->devices == 0) {
    bus->exit(bus);
#ifdef CONFIG_DEVFS_FS    
    devfs_unregister( bus->handle );
#endif /* CONFIG_DEVFS_FS */
    ret = 0;
    MOD_DEC_USE_COUNT;
    ssi_bus_count--;
  }
  return ret;
}

static struct file_operations ssi_core_fops = {
  owner:   THIS_MODULE,
};

/* SSI Core Initialization 
 */
static int __init ssi_init(void)
{
  unsigned int bus;

#ifdef CONFIG_DEVFS_FS
  if (devfs_register_chrdev( SSI_MAJOR , "ssi" , &ssi_core_fops)==-1) 
  {
    printk(KERN_ERR "SSI: ssi device already in use.\n");
    return -EBUSY;
  }
  ssi_devfs_handle = devfs_mk_dir( NULL , "ssi" , NULL );
#endif /* CONFIG_DEVFS_FS */
  ssi_bus_count=0;

  for (bus=0; bus<SSI_MAX_BUSSES; bus++) 
    ssi_busses[bus] = 0;

  return 0;
}

/* SSI Core Destruction 
 */
static void __exit ssi_exit(void)
{
#ifdef CONFIG_DEVFS_FS
  devfs_unregister_chrdev( SSI_MAJOR , "ssi" );
  devfs_unregister( ssi_devfs_handle );
#endif /* CONFIG_DEVFS_FS */
}

module_init(ssi_init);
module_exit(ssi_exit);
