/* 
 * Copyright (C) 2002 TransDimension, Inc. All Rights Reserved
 * Name: stack_urb.h
 * Author: Biao Jia		6/24/02
 * Description:
 *    URB adapter. Convert OS specific URB to otg urb.
 */
#ifndef STACK_URB_H
#define STACK_URB_H

#include <linux/usb.h>

typedef struct urb stack_urb_t;
typedef struct usb_device stack_device_t;
typedef struct usb_bus stack_bus_t;
typedef devrequest stack_setup_t;
typedef iso_packet_descriptor_t stack_urb_iso_t;

#include <stack_pipe.h>

#define	stack_urb_PipeTypeCtrl	PIPE_CONTROL
#define	stack_urb_PipeTypeBulk	PIPE_BULK
#define	stack_urb_PipeTypeIntr	PIPE_INTERRUPT
#define	stack_urb_PipeTypeIso	PIPE_ISOCHRONOUS

static inline stack_pipe_t
stack_urb_GetPipe(stack_urb_t *urb)
{
	return urb->pipe;
}

static inline int
stack_urb_HasSamePipe(stack_urb_t *urb, stack_urb_t *urb1)
{
	return urb->pipe == urb1->pipe;
}

static inline int
stack_urb_HasSameAddress(stack_urb_t *urb, stack_urb_t *urb1)
{
	return (urb->pipe ^ urb1->pipe) & ~USB_DIR_IN;
}

static inline stack_device_t *
stack_urb_GetDevice(stack_urb_t *urb)
{
	return urb->dev;
}

static inline void
stack_urb_SetDevice(stack_urb_t *urb, stack_device_t *dev)
{
	urb->dev = dev;
}

static inline int
stack_urb_IsDevice(stack_urb_t *urb, stack_device_t *dev)
{
	return dev == urb->dev;
}

static inline int
stack_urb_HasSameDevice(stack_urb_t *urb, stack_urb_t *urb1)
{
	return urb->dev == urb1->dev;
}

static inline stack_bus_t *
stack_urb_GetBus(stack_urb_t *urb)
{
	return urb->dev->bus;
}

static inline int
stack_urb_GetAddress(stack_urb_t *urb)
{
	return usb_pipedevice(urb->pipe);
}

static inline int
stack_urb_GetEndpoint(stack_urb_t *urb)
{
	return usb_pipeendpoint(urb->pipe);
}

static inline int
stack_urb_GetActualLength(stack_urb_t *urb)
{
	return urb->actual_length;
}

static inline void
stack_urb_SetActualLength(stack_urb_t *urb, int len)
{
	urb->actual_length = len;
}

static inline void
stack_urb_IncActualLength(stack_urb_t *urb, int len)
{
	urb->actual_length += len;
}

static inline int
stack_urb_GetBufferLength(stack_urb_t *urb)
{
	return urb->transfer_buffer_length;
}

static inline void *
stack_urb_GetBuffer(stack_urb_t *urb)
{
	return urb->transfer_buffer;
}

static inline int
stack_urb_GetPacketCount(stack_urb_t *urb)
{
	return urb->number_of_packets;
}

static inline int
stack_urb_GetIsoLength(stack_urb_t *urb, int packet)
{
	return urb->iso_frame_desc[packet].length;
}

static inline int
stack_urb_GetIsoOffset(stack_urb_t *urb, int packet)
{
	return urb->iso_frame_desc[packet].offset;
}

static inline stack_urb_iso_t *
stack_urb_GetIso(stack_urb_t *urb, int packet)
{
	return &urb->iso_frame_desc[packet];
}

static inline stack_urb_t *
stack_urb_GetNextUrb(stack_urb_t *urb)
{
	return urb->next;
}

static inline int
stack_urb_GetStartFrame(stack_urb_t *urb)
{
	return urb->start_frame;
}

static inline void
stack_urb_SetStartFrame(stack_urb_t *urb, int frame)
{
	urb->start_frame = frame;
}

static inline void
stack_urb_MaskStartFrame(stack_urb_t *urb, int mask)
{
	urb->start_frame &= mask;
}

static inline void
stack_urb_SetNextStartFrame(stack_urb_t *urb, stack_urb_t *prev)
{
	urb->start_frame = prev->start_frame + prev->number_of_packets;
}

static inline int
stack_urb_GetPipeType(stack_urb_t *urb)
{
	return usb_pipetype(urb->pipe);
}

static inline int
stack_urb_IsCtrlPipe(stack_urb_t *urb)
{
	return usb_pipecontrol(urb->pipe);
}

static inline int
stack_urb_IsBulkPipe(stack_urb_t *urb)
{
	return usb_pipebulk(urb->pipe);
}

static inline int
stack_urb_IsIntrPipe(stack_urb_t *urb)
{
	return usb_pipeint(urb->pipe);
}

static inline int
stack_urb_IsIsoPipe(stack_urb_t *urb)
{
	return usb_pipeisoc(urb->pipe);
}

static inline void
stack_IncDeviceUse(stack_device_t *dev)
{
	usb_inc_dev_use(dev);
}

static inline void
stack_DecDeviceUse(stack_device_t *dev)
{
	usb_dec_dev_use(dev);
}

static inline int
stack_urb_GetStatus(stack_urb_t *urb)
{
	return urb->status;
}

static inline void
stack_urb_SetStatus(stack_urb_t *urb, int status)
{
	urb->status = status;
}

static inline int
stack_urb_HasBandwidth(stack_urb_t *urb)
{
	return urb->bandwidth;
}

static inline int
stack_urb_CheckBandwidth(stack_urb_t *urb)
{
	return usb_check_bandwidth(urb->dev, urb);
}

static inline void
stack_urb_ClaimBandwidth(stack_urb_t *urb, int bustime, int is_iso)
{
	return usb_claim_bandwidth(urb->dev, urb, bustime, is_iso);
}

static inline void
stack_urb_ReleaseBandwidth(stack_urb_t *urb, int is_iso)
{
	return usb_release_bandwidth(urb->dev, urb, is_iso);
}

static inline int
stack_urb_IsValidInterval(stack_urb_t *urb)
{
	return urb->interval >= 0 && urb->interval <= 255;
}

static inline int
stack_urb_GetIntrInterval(stack_urb_t *urb)
{
	return urb->interval;
}

static inline void
stack_urb_SetIntrInterval(stack_urb_t *urb, int val)
{
	urb->interval = val;
}

static inline usb_complete_t
stack_urb_HasCallBack(stack_urb_t *urb)
{
	return urb->complete;
}

static inline void
stack_urb_CallBack(stack_urb_t *urb)
{
	urb->complete(urb);
}

static inline int
stack_urb_IsIsoASAP(stack_urb_t *urb)
{
	return urb->transfer_flags & USB_ISO_ASAP;
}

static inline int
stack_urb_IsShortPacketAllowed(stack_urb_t *urb)
{
	return !(urb->transfer_flags & USB_DISABLE_SPD);
}

static inline int
stack_urb_IsBulkQueued(stack_urb_t *urb)
{
	return urb->transfer_flags & USB_QUEUE_BULK;
}

static inline int
stack_urb_IsErrorStatus(stack_urb_t *urb)
{
	return urb->status && urb->status != -EINPROGRESS;
}

static inline void
stack_urb_AddFlagTimeout(stack_urb_t *urb)
{
	urb->transfer_flags |= USB_TIMEOUT_KILLED;
}

static inline void *
stack_urb_GetContext(stack_urb_t *urb)
{
	return urb->context;
}

static inline void
stack_urb_SetErrorCount(stack_urb_t *urb, int count)
{
	urb->error_count = count;
}

static inline void
stack_urb_IncErrorCount(stack_urb_t *urb)
{
	urb->error_count++;
}

static inline int
stack_urb_GetMaxPacketSize(stack_urb_t *urb, stack_pipe_t pipe)
{
	return usb_maxpacket(urb->dev, pipe, stack_pipe_IsOut(pipe));
}

static inline int
stack_urb_IsHalted(stack_urb_t *urb, stack_pipe_t pipe)
{
	return usb_endpoint_halted(urb->dev, usb_pipeendpoint(pipe), stack_pipe_IsOut(pipe));
}

static inline void
stack_urb_Halt(stack_urb_t *urb)
{
	usb_endpoint_halt(urb->dev, usb_pipeendpoint(urb->pipe), stack_pipe_IsOut(urb->pipe));
}

static inline int
stack_urb_GetToggle(stack_urb_t *urb, stack_pipe_t pipe)
{
	return usb_gettoggle(urb->dev, usb_pipeendpoint(pipe), usb_pipeout(pipe));
}

static inline void
stack_urb_SetToggle(stack_urb_t *urb, int toggle)
{
	usb_settoggle
	(
		urb->dev,
		usb_pipeendpoint(urb->pipe),
		usb_pipeout(urb->pipe),
		toggle
	);
}

static inline stack_setup_t *
stack_urb_GetSetup(stack_urb_t *urb)
{
	return (stack_setup_t *)urb->setup_packet;
}

static inline int
stack_urb_GetReqestType(stack_setup_t *setup)
{
	return setup->requesttype;
}

static inline int
stack_urb_GetReqest(stack_setup_t *setup)
{
	return setup->request;
}

static inline int
stack_urb_GetReqestValue(stack_setup_t *setup)
{
	return setup->value;
}

static inline int
stack_urb_GetReqestIndex(stack_setup_t *setup)
{
	return setup->index;
}

static inline int
stack_urb_GetReqestLength(stack_setup_t *setup)
{
	return setup->length;
}

static inline int
stack_urb_GetIsoActualLength(stack_urb_iso_t *iso)
{
	return iso->actual_length;
}

static inline void
stack_urb_SetIsoActualLength(stack_urb_iso_t *iso, int len)
{
	iso->actual_length = len;
}

static inline void
stack_urb_CopyIsoLength(stack_urb_iso_t *iso)
{
	iso->actual_length = iso->length;
}

static inline int
stack_urb_GetIsoStatus(stack_urb_iso_t *iso)
{
	return iso->status;
}

static inline void
stack_urb_SetIsoStatus(stack_urb_iso_t *iso, int status)
{
	iso->status = status;
}

static inline int
stack_urb_IsOneStepSetup(void)
{
	return 1;
}

#include <otg_linux.h>

#endif /* #ifndef STACK_URB_H */
